@@ -1,15 +1,18 @@
 '''
 Runnable
 ========
-
 '''
 
 from jnius import PythonJavaClass, java_method, autoclass
 from android.config import JAVA_NAMESPACE
 
-# reference to the activity
+# Reference to the activity
 _PythonActivity = autoclass(JAVA_NAMESPACE + '.PythonActivity')
 
+# Cache of functions table. In older Android versions the number of JNI references
+# is limited, so by caching them we avoid running out.
+__functionstable__ = {}
+
 
 class Runnable(PythonJavaClass):
     '''Wrapper around Java Runnable class. This class can be used to schedule a
@@ -20,7 +23,7 @@ class Runnable(PythonJavaClass):
     __runnables__ = []
 
     def __init__(self, func):
-        super(Runnable, self).__init__()
+        super().__init__()
         self.func = func
 
     def __call__(self, *args, **kwargs):
@@ -33,7 +36,7 @@ class Runnable(PythonJavaClass):
     def run(self):
         try:
             self.func(*self.args, **self.kwargs)
-        except:
+        except:  # noqa E722
             import traceback
             traceback.print_exc()
 
@@ -44,6 +47,12 @@ def run_on_ui_thread(f):
     '''Decorator to create automatically a :class:`Runnable` object with the
     function. The function will be delayed and call into the Activity thread.
     '''
+    if f not in __functionstable__:
+        rfunction = Runnable(f)  # store the runnable function
+        __functionstable__[f] = {"rfunction": rfunction}
+    rfunction = __functionstable__[f]["rfunction"]
+
     def f2(*args, **kwargs):
-        Runnable(f)(*args, **kwargs)
+        rfunction(*args, **kwargs)
+
     return f2
