@@ -19,7 +19,7 @@ class PyZMQRecipe(CythonRecipe):
                    '-Izmq/devices']
 
     def get_recipe_env(self, arch=None):
-        env = super(PyZMQRecipe, self).get_recipe_env(arch)
+        env = super().get_recipe_env(arch)
         # TODO: fix hardcoded path
         # This is required to prevent issue with _io.so import.
         # hostpython = self.get_recipe('hostpython2', self.ctx)
@@ -43,9 +43,9 @@ class PyZMQRecipe(CythonRecipe):
 [global]
 zmq_prefix = {}
 skip_check_zmq = True
-""".format(libzmq_prefix))
+""".format(libzmq_prefix).encode())
 
-        return super(PyZMQRecipe, self).build_cython_components(arch)
+        return super().build_cython_components(arch)
 
         with current_directory(self.get_build_dir(arch.arch)):
             hostpython = sh.Command(self.hostpython_location)
